@@ -1,3 +1,4 @@
+import anydbm
 import cPickle
 import logging
 import os.path
@@ -26,9 +27,11 @@ def container_registry(name):
     return _cls_registry(name, 'Container')
 
 def _cls_registry(name, clsname):
-    if name.startswith('ext:'):
-        name = name[4:]
-        modname = "beaker.ext." + name 
+    if name.startswith('ext:') \
+            or name in ['memcached', 'database', 'sqla', 'google']:
+        if name.startswith('ext:'):
+            name = name[4:]
+        modname = "beaker.ext." + name
         mod = getattr(__import__(modname).ext, name)
     else:
         mod = sys.modules[__name__]
@@ -114,8 +117,12 @@ class NamespaceManager(object):
         acquire/release supports reentrant/nested operation."""
         
         self.do_acquire_read_lock()
-        self.open('r', checkcount = True)
-        
+        try:
+            self.open('r', checkcount = True)
+        except:
+            self.do_release_read_lock()
+            raise
+            
     def release_read_lock(self):
         """releases the read lock for this namespace, and possibly
         closes the datasource, if it was opened as a product of
@@ -123,8 +130,10 @@ class NamespaceManager(object):
 
         acquire/release supports reentrant/nested operation."""
     
-        self.close(checkcount = True)
-        self.do_release_read_lock()
+        try:
+            self.close(checkcount = True)
+        finally:
+            self.do_release_read_lock()
         
     def acquire_write_lock(self, wait = True): 
         """acquires a write lock for this namespace, and 
@@ -134,9 +143,13 @@ class NamespaceManager(object):
         acquire/release supports reentrant/nested operation."""
         
         r = self.do_acquire_write_lock(wait)
-        if (wait or r): self.open('c', checkcount = True)
-        return r
-        
+        try:
+            if (wait or r): self.open('c', checkcount = True)
+            return r
+        except:
+            self.do_release_write_lock()
+            raise
+            
     def release_write_lock(self): 
         """releases the write lock for this namespace, and possibly
         closes the datasource, if it was opened as a product of
@@ -144,8 +157,10 @@ class NamespaceManager(object):
 
         acquire/release supports reentrant/nested operation."""
 
-        self.close(checkcount = True)
-        self.do_release_write_lock()
+        try:
+            self.close(checkcount = True)
+        finally:
+            self.do_release_write_lock()
 
     def open(self, flags, checkcount = False):
         """opens the datasource for this namespace.
@@ -528,7 +543,6 @@ class DBMNamespaceManager(NamespaceManager):
             self.lock_dir = data_dir + "/container_dbm_lock"
 
         if dbmmodule is None:
-            import anydbm
             self.dbmmodule = anydbm
         else:
             self.dbmmodule = dbmmodule
@@ -699,7 +713,10 @@ class FileNamespaceManager(NamespaceManager):
     def do_open(self, flags):
         if self.file_exists(self.file):
             fh = open(self.file, 'rb')
-            self.hash = cPickle.load(fh)
+            try:
+                self.hash = cPickle.load(fh)
+            except (IOError, OSError, EOFError, cPickle.PickleError):
+                pass
             fh.close()
         self.flags = flags
         
