@@ -28,7 +28,8 @@ class MemcachedNamespaceManager(NamespaceManager):
         
         verify_directory(self.lock_dir)            
         
-        self.mc = MemcachedNamespaceManager.clients.get(url, lambda: memcache.Client(url.split(';'), debug=0))
+        self.mc = MemcachedNamespaceManager.clients.get(url, 
+            lambda: memcache.Client(url.split(';'), debug=0))
 
     # memcached does its own locking.  override our own stuff
     def do_acquire_read_lock(self): pass
@@ -42,19 +43,23 @@ class MemcachedNamespaceManager(NamespaceManager):
     def close(self, *args, **params):pass
 
     def __getitem__(self, key):
-        key = key.replace(' ', '\302\267')
-        value = self.mc.get(self.namespace + "_" + key)
-        if value is None:
+        ns_key = self.namespace + '_' + key.replace(' ', '\302\267')
+        all_key = self.namespace + ':keys'
+        keys = [ns_key, all_key]
+        key_dict = self.mc.get_multi(keys)
+        if ns_key not in key_dict:
             raise KeyError(key)
-        return value
+        return key_dict[ns_key]
 
     def __contains__(self, key):
-        key = key.replace(' ', '\302\267')
-        return self.mc.get(self.namespace + "_" + key) is not None
+        return self.has_key(key)
 
     def has_key(self, key):
-        key = key.replace(' ', '\302\267')
-        return self.mc.get(self.namespace + "_" + key) is not None
+        ns_key = self.namespace + '_' + key.replace(' ', '\302\267')
+        all_key = self.namespace + ':keys'
+        keys = [ns_key, all_key]
+        key_dict = self.mc.get_multi(keys)
+        return ns_key in key_dict
 
     def __setitem__(self, key, value):
         key = key.replace(' ', '\302\267')
@@ -66,28 +71,33 @@ class MemcachedNamespaceManager(NamespaceManager):
         self.mc.set(self.namespace + "_" + key, value)
 
     def __delitem__(self, key):
-        key = key.replace(' ', '\302\267')
-        keys = self.mc.get(self.namespace + ':keys')
-        try:
-            del keys[key]
-            self.mc.delete(self.namespace + "_" + key)
-            self.mc.set(self.namespace + ':keys', keys)
-        except KeyError:
-            raise
+        cache_key = key.replace(' ', '\302\267')
+        ns_key = self.namespace + '_' + cache_key
+        all_key = self.namespace + ':keys'
+        keys = [ns_key, all_key]
+        key_dict = self.mc.get_multi(keys)
+        if ns_key in key_dict:
+            self.mc.delete(ns_key)
+            mem_keys = key_dict.get(all_key, {})
+            if cache_key in mem_keys:
+                del mem_keys[cache_key]
+                self.mc.set(all_key, mem_keys)
+        else:
+            raise KeyError
 
     def do_remove(self):
         keys = self.mc.get(self.namespace + ':keys')
         if keys is not None:
-            for key in keys:
-                self.mc.delete(self.namespace + '_' + key)
-            self.mc.delete(self.namespace + ':keys')
+            delete_keys = [self.namespace + '_' + x for x in keys]
+            delete_keys.append(self.namespace + ':keys')
+            self.mc.delete_multi(delete_keys)
     
     def keys(self):
         keys = self.mc.get(self.namespace + ':keys')
         if keys is None:
             return []
         else:
-            return [x.replace('\302\267') for x in keys.keys()]
+            return [x.replace('\302\267', ' ') for x in keys.keys()]
 
 class MemcachedContainer(Container):
 
