@@ -21,6 +21,56 @@ except NameError:
 
 from beaker.converters import asbool
 
+try:
+    from base64 import b64encode, b64decode
+except ImportError:
+    import binascii
+
+    _translation = [chr(_x) for _x in range(256)]
+
+    # From Python 2.5 base64.py
+    def _translate(s, altchars):
+        translation = _translation[:]
+        for k, v in altchars.items():
+            translation[ord(k)] = v
+        return s.translate(''.join(translation))
+
+    def b64encode(s, altchars=None):
+        """Encode a string using Base64.
+
+        s is the string to encode.  Optional altchars must be a string of at least
+        length 2 (additional characters are ignored) which specifies an
+        alternative alphabet for the '+' and '/' characters.  This allows an
+        application to e.g. generate url or filesystem safe Base64 strings.
+
+        The encoded string is returned.
+        """
+        # Strip off the trailing newline
+        encoded = binascii.b2a_base64(s)[:-1]
+        if altchars is not None:
+            return _translate(encoded, {'+': altchars[0], '/': altchars[1]})
+        return encoded
+
+    def b64decode(s, altchars=None):
+        """Decode a Base64 encoded string.
+
+        s is the string to decode.  Optional altchars must be a string of at least
+        length 2 (additional characters are ignored) which specifies the
+        alternative alphabet used instead of the '+' and '/' characters.
+
+        The decoded string is returned.  A TypeError is raised if s were
+        incorrectly padded or if there are non-alphabet characters present in the
+        string.
+        """
+        if altchars is not None:
+            s = _translate(s, {altchars[0]: '+', altchars[1]: '/'})
+        try:
+            return binascii.a2b_base64(s)
+        except binascii.Error, msg:
+            # Transform this exception for consistency
+            raise TypeError(msg)
+
+
 def verify_directory(dir):
     """verifies and creates a directory.  tries to
     ignore collisions with other threads and processes."""
@@ -172,6 +222,8 @@ def encoded_path(root, identifiers, extension = ".enc", depth = 3, digest = True
 
     if digest:
         ident = sha.new(ident).hexdigest()
+    
+    ident = os.path.basename(ident)
 
     tokens = []
     for d in range(1, depth):
@@ -213,9 +265,13 @@ def coerce_session_params(params):
         ('lock_dir', (str,), "lock_dir must be a string referring to a directory."),
         ('type', (str, types.NoneType), "Session type must be a string."),
         ('cookie_expires', (bool, datetime, timedelta), "Cookie expires was not a boolean, datetime, or timedelta instance."),
+        ('cookie_domain', (str, types.NoneType), "Cookie domain must be a string."),
         ('id', (str,), "Session id must be a string."),
         ('key', (str,), "Session key must be a string."),
         ('secret', (str, types.NoneType), "Session secret must be a string."),
+        ('validate_key', (str, types.NoneType), "Session encrypt_key must be a string."),
+        ('encrypt_key', (str, types.NoneType), "Session validate_key must be a string."),
+        ('secure', (bool, types.NoneType), "Session secure must be a boolean."),
         ('timeout', (int, types.NoneType), "Session timeout must be an integer."),
     ]
     return verify_rules(params, rules)
