@@ -1,9 +1,14 @@
 # coding: utf-8
-from paste.fixture import *
+from beaker.cache import clsmap, Cache
 from beaker.middleware import CacheMiddleware
-from beaker.cache import Cache
+from nose import SkipTest
+from webtest import TestApp
 
-mc_url = '127.0.0.1'
+if 'ext:memcached' not in clsmap:
+    raise SkipTest("'memcache' or 'cmemcache' is not installed, can't test "
+                   "memcached backend")
+
+mc_url = '127.0.0.1:11211'
 
 def simple_app(environ, start_response):
     extra_args = {}
@@ -24,6 +29,26 @@ def simple_app(environ, start_response):
     start_response('200 OK', [('Content-type', 'text/plain')])
     return ['The current value is: %s' % cache.get_value('value')]
 
+def using_none_app(environ, start_response):
+    extra_args = {}
+    clear = False
+    if environ.get('beaker.clear'):
+        clear = True
+    extra_args['type'] = 'ext:memcached'
+    extra_args['url'] = mc_url
+    extra_args['data_dir'] = './cache'
+    cache = environ['beaker.cache'].get_cache('testcache', **extra_args)
+    if clear:
+        cache.clear()
+    try:
+        value = cache.get_value('value')
+    except:
+        value = 10
+    cache.set_value('value', None)
+    start_response('200 OK', [('Content-type', 'text/plain')])
+    return ['The current value is: %s' % value]
+
+
 def cache_manager_app(environ, start_response):
     cm = environ['beaker.cache']
     cm.get_cache('test')['test_key'] = 'test value'
@@ -51,6 +76,40 @@ def test_has_key():
     cache.remove_value("test")
     assert not cache.has_key("test")
 
+def test_dropping_keys():
+    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
+    cache.set_value('test', 20)
+    cache.set_value('fred', 10)
+    assert cache.has_key('test')
+    assert 'test' in cache
+    assert cache.has_key('fred')
+    
+    # Directly nuke the actual key, to simulate it being removed by memcached
+    cache._containers['test'].namespacemanager.mc.delete('test_test')
+    assert not cache.has_key('test')
+    assert cache.has_key('fred')
+    
+    # Nuke the keys dict, it might die, who knows
+    cache._containers['fred'].namespacemanager.mc.delete('test:keys')
+    assert cache.has_key('fred')
+    
+    # And we still need clear to work, even if it won't work well
+    cache.clear()
+
+def test_deleting_keys():
+    cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
+    cache.set_value('test', 20)
+    
+    # Nuke the keys dict, it might die, who knows
+    cache._containers['test'].namespacemanager.mc.delete('test:keys')
+    
+    assert cache.has_key('test')
+    
+    # make sure we can still delete keys even though our keys dict got nuked
+    del cache['test']
+    
+    assert not cache.has_key('test')
+
 def test_has_key_multicache():
     cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
     o = object()
@@ -109,4 +168,10 @@ def test_cache_manager():
     res = app.get('/')
     assert 'test_key is: test value' in res
     assert 'test_key cleared' in res
-    
+
+def test_store_none():
+    app = TestApp(CacheMiddleware(using_none_app))
+    res = app.get('/', extra_environ={'beaker.clear':True})
+    assert 'current value is: 10' in res
+    res = app.get('/')
+    assert 'current value is: None' in res
